-- This filters worker versions to only include those used in any of the kinds
-- of ML results we have.  Doing it using LEFT JOIN would require 10 joins and
-- fills up the RAM.  Adding DISTINCT to all the SELECT queries of the UNION
-- slows this query down by ~20%.  Using multiple INs instead of a UNION makes
-- this query twice as slow.
SELECT DISTINCT version.id, worker.name, worker.slug, workertype.slug, version.version, revision.hash, repository.url
FROM process_workerrun run
INNER JOIN process_workerversion version ON (run.version_id = version.id)
INNER JOIN process_worker worker ON (version.worker_id = worker.id)
INNER JOIN process_workertype workertype ON (worker.type_id = workertype.id)
LEFT JOIN process_repository repository ON (worker.repository_id = repository.id)
LEFT JOIN process_revision revision ON (version.revision_id = revision.id)
WHERE run.id IN (
    SELECT worker_run_id FROM documents_element WHERE corpus_id = '{corpus_id}'::uuid
UNION
    SELECT worker_run_id FROM documents_entity WHERE corpus_id = '{corpus_id}'::uuid
UNION
    SELECT classification.worker_run_id
    FROM documents_classification classification
    INNER JOIN documents_element element ON (element.id = classification.element_id)
    WHERE element.corpus_id = '{corpus_id}'::uuid
UNION
    SELECT transcription.worker_run_id
    FROM documents_transcription transcription
    INNER JOIN documents_element element ON (element.id = transcription.element_id)
    WHERE element.corpus_id = '{corpus_id}'::uuid
UNION
    SELECT te.worker_run_id
    FROM documents_transcriptionentity te
    INNER JOIN documents_entity entity ON (te.entity_id = entity.id)
    WHERE entity.corpus_id = '{corpus_id}'::uuid
UNION
    SELECT md.worker_run_id
    FROM documents_metadata md
    INNER JOIN documents_element element ON (md.element_id = element.id)
    WHERE element.corpus_id = '{corpus_id}'::uuid
)
