import json
from hashlib import md5

from django.conf import settings
from django.db.models import ExpressionWrapper, F, TextField, Value
from django.db.models.functions import Concat, NullIf
from django.utils.module_loading import import_string

from arkindex.project.tools import RTrimChr

get_default_farm = (
    import_string(settings.GET_DEFAULT_FARM)
    if getattr(settings, "GET_DEFAULT_FARM", None)
    else lambda: None
)


def hash_object(object):
    object_json = json.dumps(object, sort_keys=True).encode("utf-8")
    return md5(object_json).hexdigest()


def annotate_image_url(field="image"):
    """
    Build the image URL by concatenating the server's URL to the image's path
    Server URLs might end in a slash, but not all the time,
    so we strip any trailing slashes and add our own.
    This uses NullIf as this concatenation will return '/' if there is no zone on the element.
    """
    slash = Value("/", output_field=TextField())
    return NullIf(
        Concat(
            RTrimChr(f"{field}__server__url", slash),
            slash,
            ExpressionWrapper(F(f"{field}__path"), output_field=TextField()),
        ),
        slash,
    )
